
/* This program tests how to create and use "external links" in
 * HDF5.
 *
 * External links point from one HDF5 file to an object (Group, Dataset, or
 * committed Datatype) in another file.
 */

#include "hdf5.h"
#include <string.h>

#define SOURCE_FILE "extlink_source.h5"
#define TARGET_FILE "extlink_target.h5"

#define PREFIX_SOURCE_FILE "extlink_prefix_source.h5"

#define SOFT_LINK_FILE "soft_link.h5"
#define SOFT_LINK_NAME "soft_link_to_group"
#define UD_SOFT_LINK_NAME "ud_soft_link"
#define TARGET_GROUP "target_group"

#define UD_SOFT_CLASS 65

#define HARD_LINK_FILE "hard_link.h5"
#define HARD_LINK_NAME "hard_link_to_group"
#define UD_HARD_LINK_NAME "ud_hard_link"

#define UD_HARD_CLASS 66

#define PLIST_LINK_PROP "plist_link_prop"
#define UD_PLIST_CLASS 66



/* External link prefix example from https://www.hdfgroup.org/ftp/HDF5/current/src/unpacked/examples/h5_extlink.c
 *
 * Uses a group access property list to set a "prefix" for the filenames
 * accessed through an external link.
 *
 * Group access property lists inherit from link access property lists;
 * the external link prefix property is actually a property of LAPLs.
 *
 * This example requires a "red" directory directory to exist
 * where it is run (so to run this example on Unix, first mkdir red and mkdir
 * blue).
 */
static void extlink_prefix_example(void)
{
    hid_t source_file_id, red_file_id, blue_file_id;
    hid_t group_id, group2_id;
    hid_t gapl_id;

    source_file_id = H5Fcreate(PREFIX_SOURCE_FILE, H5F_ACC_TRUNC, H5P_DEFAULT, H5P_DEFAULT);
    red_file_id = H5Fcreate("red/prefix_target.h5", H5F_ACC_TRUNC, H5P_DEFAULT, H5P_DEFAULT);
    blue_file_id = H5Fcreate("blue/prefix_target.h5", H5F_ACC_TRUNC, H5P_DEFAULT, H5P_DEFAULT);

    if(red_file_id < 0 || blue_file_id < 0)
      printf("This test requires a directory 'red' to exist. Did you forget to create it?\n");

    H5Lcreate_external("prefix_target.h5", "/", source_file_id, "ext_link", H5P_DEFAULT, H5P_DEFAULT);
    printf("%s:%s --> %s:%s\n", PREFIX_SOURCE_FILE, "/ext_link", "prefix_target.h5","/");

    gapl_id = H5Pcreate(H5P_GROUP_ACCESS);
    H5Pset_elink_prefix(gapl_id, "red/");
    printf("%s is in %s\n",  "prefix_target.h5","red/");
    
    group_id = H5Gopen2(source_file_id, "ext_link", gapl_id);

    group2_id = H5Gcreate2(group_id, "pink", H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);

    H5Gclose(group2_id);
    H5Gclose(group_id);
    H5Pclose(gapl_id);
    H5Fclose(red_file_id);
    H5Fclose(source_file_id);
}

static void extlink_traverse(void)
{
    hid_t link_file_id;
    hid_t group_id, group2_id;
    hid_t gapl_id;

    link_file_id = H5Fopen(PREFIX_SOURCE_FILE, H5F_ACC_RDONLY, H5P_DEFAULT);
    
    if(link_file_id < 0)
      printf("This test requires >>%s<< to exist. \n", PREFIX_SOURCE_FILE);
    else
      printf(">>%s<< Ready. \n", PREFIX_SOURCE_FILE);
    
    group_id = H5Gopen2(link_file_id, "ext_link", H5P_DEFAULT);
    if(group_id<1){
      printf("opening ext_link failed!\n");
      H5Gclose(group_id);
      H5Fclose(link_file_id);
      return;
    }
 
    gapl_id = H5Pcreate(H5P_GROUP_ACCESS);
    char name[128];
    ssize_t res =  H5Pget_elink_prefix( gapl_id, &name, 128 );
    printf("ext_link points to %s\n",name);
    group2_id = H5Gopen2(group_id, "pink", H5P_DEFAULT);

    if(group2_id<1)
      printf("traversal to pink failed!\n");
    else
      printf("SUCCESS ... pink is there!\n");
    
    H5Gclose(group2_id);
    H5Gclose(group_id);
    H5Fclose(link_file_id);

    return;
}

/* Main function
 *
 * Invokes the example functions.
 */
 int
main(void)
{

    printf("[TEST] Testing external link prefixes.\n");
    extlink_prefix_example();

    printf("[TEST] Traversal through external prefixed link.\n");
    extlink_traverse();

    return 0;
}


